/*
 * Copyright (C) 2010 The Android-x86 Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Yi Sun <beyounn@gmail.com>
 *  22-07-2013, Gigaset Communications GmbH
 * Feature added to configure ethernet ip
 */

package com.maxwell.settings.ethernet;


import java.util.List;

import com.android.settings.R;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.view.View.OnClickListener;
import android.content.Intent;
import android.net.NetworkInfo;
import com.maxwell.net.ethernet.EthernetManager;
import com.maxwell.net.ethernet.EthernetDevInfo;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.os.SystemProperties;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Button;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TextView;
import android.util.Slog;
import java.util.regex.*;
import android.widget.Toast;
import android.net.ConnectivityManager;
import android.net.NetworkUtils;
import java.net.InetAddress;

public class EthernetConfigDialog {
    private final String TAG = "EtherenetSettings";
    private final String INTERFACE_NAME = "eth0";
    private final int TYPE_ETHERNET = 9;
    private static final boolean localLOGV = false;

    
    private RadioButton mConTypeDhcp;
    private RadioButton mConTypeManual;
    private RadioButton mNetTypeIPv4;
    private RadioButton mNetTypeIPv6;
    private EditText mIpaddr;
    private EditText mDns;
    private EditText mGw;
    private EditText mMask;
    private EditText mPrefixLen;
    private TextView mMasktv;
    private TextView mPrefixLentv;
    private Button mCancel;
    private Button mSave;
    private Context mContext = null;
    private Dialog mDialog = null;

    private EthernetLayer mEthLayer;
    private EthernetManager mEthManager;
    private ConnectivityManager mConnManager;
    private EthernetDevInfo mEthInfo;

    /*Static IPV4 Validation using regular expression*/
    private static final String PATTERN =
        "^([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\." +
        "([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\." +
        "([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\." +
        "([01]?\\d\\d?|2[0-4]\\d|25[0-5])$";

    private final String ETHERNET_IPV4 = "ipv4";
    private final String ETHERNET_IPV6 = "ipv6";

    public EthernetConfigDialog(Context context){
        mContext = context;
        mEthLayer = new EthernetLayer(this);
        mEthManager= (EthernetManager) mContext.getSystemService(Context.ETHERNET_SERVICE);
        mConnManager = (ConnectivityManager) mContext.getSystemService(Context.CONNECTIVITY_SERVICE);
    }

    /*Construct Dialog for showing the Ethernet Configuration*/
    public Dialog buildDialogContent(Context context) {
        mDialog = new Dialog(context);
        mDialog.setTitle(R.string.eth_config_title);
        mDialog.setCancelable(true);
        mDialog.setContentView(R.layout.eth_configure);

        mNetTypeIPv4 = (RadioButton) mDialog.findViewById(R.id.ipv4_radio);
        mNetTypeIPv6 = (RadioButton) mDialog.findViewById(R.id.ipv6_radio);
        mConTypeDhcp = (RadioButton) mDialog.findViewById(R.id.dhcp_radio);
        mConTypeManual = (RadioButton) mDialog.findViewById(R.id.manual_radio);
        mIpaddr = (EditText) mDialog.findViewById(R.id.ipaddr_edit);
        mMask = (EditText) mDialog.findViewById(R.id.netmask_edit);
        mDns = (EditText) mDialog.findViewById(R.id.eth_dns_edit);
        mGw = (EditText) mDialog.findViewById(R.id.eth_gw_edit);
        mPrefixLen = (EditText) mDialog.findViewById(R.id.prefixlength_edit);
        mMasktv = (TextView) mDialog.findViewById(R.id.netmask_text);
        mPrefixLentv = (TextView) mDialog.findViewById(R.id.prefixlength_text);
        mCancel = (Button) mDialog.findViewById(R.id.cancel_button);
        mSave = (Button) mDialog.findViewById(R.id.save_button);
        mCancel.setOnClickListener(mCancelHandler);
        mSave.setOnClickListener(mSaveHandler);

        mNetTypeIPv4.setOnClickListener(new RadioButton.OnClickListener() {
            public void onClick(View v) {
                mConTypeDhcp.setChecked(true);
                dhcpClicked();
            }
        });

        mNetTypeIPv6.setOnClickListener(new RadioButton.OnClickListener() {
            public void onClick(View v) {
                mConTypeDhcp.setChecked(true);
                dhcpClicked();
            }
        });

        mConTypeManual.setOnClickListener(new RadioButton.OnClickListener() {
            public void onClick(View v) {
                manualClicked();
            }
        });

        mConTypeDhcp.setOnClickListener(new RadioButton.OnClickListener() {
            public void onClick(View v) {
                dhcpClicked();
            }
        });

	mEthInfo = mEthManager.getSavedConfig();

        if(null != mEthInfo && null != mEthInfo.getNetType()) {
            if(mEthInfo.getNetType().equals(ETHERNET_IPV4)) { /*IPV4 case*/
                mNetTypeIPv4.setChecked(true);
                mNetTypeIPv6.setChecked(false);
	        if (mEthInfo.getConnectMode().equals(EthernetDevInfo.ETHERNET_CONN_MODE_DHCP)) { /*IPV4->dhcp case*/
                    setDhcp();
	        } else { /*IPV4->static case*/
                    setManual(); 
	        }
	        mMask.setText(mEthInfo.getNetMask());
            } else if(mEthInfo.getNetType().equals(ETHERNET_IPV6)) {  /*IPV6 case*/
                mNetTypeIPv4.setChecked(false);
                mNetTypeIPv6.setChecked(true);
	        if (mEthInfo.getConnectMode().equals(EthernetDevInfo.ETHERNET_CONN_MODE_DHCP)) { /*IPV6->dhcp case*/
                    setDhcp();
	        } else { /*IPV6->static case*/
                    setManual(); 
	        }
                int prefixlen = mEthInfo.getPrefixLength();
                if(-1 != prefixlen) {
                    mPrefixLen.setText(Integer.toString(prefixlen));
                }else{
                    mPrefixLen.setText("");
                }
            }

	    mIpaddr.setText(mEthInfo.getIpAddress());
	    mGw.setText(mEthInfo.getRouteAddr());
	    mDns.setText(mEthInfo.getDnsAddr());
        } else {
            mEthManager.setDefaultConf();
        }
        return mDialog;
    }

    /*
     * Enable Netmask and disable prefix length
     */
    private void prepareIPv4(boolean setStatus) {
        if(mPrefixLen.getVisibility() == View.VISIBLE) {
            mPrefixLentv.setVisibility(View.GONE);
            mPrefixLen.setVisibility(View.GONE);
         }

	if(mMask.getVisibility() == View.GONE) {
	    mMasktv.setVisibility(View.VISIBLE);  
	    mMask.setVisibility(View.VISIBLE);
	    mMask.setEnabled(setStatus);
	    mMask.setText("");
	} else {
	    mMask.setEnabled(setStatus);
	    mMask.setText("");
	}
    }

    /*
     * Enable Prefix length and disable Netmask
     */
    private void prepareIPv6(boolean setStatus) {
	if(mMask.getVisibility() == View.VISIBLE) {
	    mMasktv.setVisibility(View.GONE); 
	    mMask.setVisibility(View.GONE);
	}

	if(mPrefixLen.getVisibility() == View.GONE) {
	    mPrefixLentv.setVisibility(View.VISIBLE);  
	    mPrefixLen.setVisibility(View.VISIBLE);
	    mPrefixLen.setEnabled(setStatus);
	    mPrefixLen.setText("");
	} else {
	    mPrefixLen.setEnabled(setStatus);
	    mPrefixLen.setText("");
	}
    }

    /*
     * Dhcp case handler
     */
    private void dhcpClicked() {
        mIpaddr.setEnabled(false);
	mDns.setEnabled(false);
	mGw.setEnabled(false);

        if(mNetTypeIPv4.isChecked()) {
            prepareIPv4(false); 
        } else { 
            prepareIPv6(false); 
        } 

	mIpaddr.setText("");
	mDns.setText("");
	mGw.setText("");
    } 

    /*
     * Static case handler
     */
    private void manualClicked() {
        mIpaddr.setEnabled(true);
	mDns.setEnabled(true);
	mGw.setEnabled(true);

        if(mNetTypeIPv4.isChecked()) {
            prepareIPv4(true); 
        } else {
            prepareIPv6(true);
        } 
	mIpaddr.setText("");
	mDns.setText("");
	mGw.setText("");
    } 

    /*
     * setting dhcp information in the UI
     */
    private void setDhcp() {
        mConTypeDhcp.setChecked(true);
	mConTypeManual.setChecked(false);
	mIpaddr.setEnabled(false);
	mDns.setEnabled(false);
	mGw.setEnabled(false);
        if(mNetTypeIPv4.isChecked()) {
            prepareIPv4(false);
        } else {
            prepareIPv6(false);
        } 
    }

    /*
     * setting the static information in the UI
     */
    private void setManual() {
        mConTypeDhcp.setChecked(false);
	mConTypeManual.setChecked(true);
	mIpaddr.setEnabled(true);
	mDns.setEnabled(true);
	mGw.setEnabled(true);
        if(mNetTypeIPv4.isChecked()) {
            prepareIPv4(true);
        } else {
            prepareIPv6(true);
        } 
    }

    /*Cancel Button Handler*/
    OnClickListener mCancelHandler = new OnClickListener() {
        public void onClick(View v) {
            mDialog.dismiss();
        }
    };

    /*Save Button Handler*/
    OnClickListener mSaveHandler = new OnClickListener() {
        public void onClick(View v) {
            if(mNetTypeIPv4.isChecked() && mConTypeDhcp.isChecked()) {
                startThread();
                Toast.makeText(mContext,  mContext.getResources().getString(R.string.DHCP_IP_CONFIGURE_MSG), Toast.LENGTH_LONG).show();
                mDialog.dismiss();
            } else if(mNetTypeIPv6.isChecked() && mConTypeDhcp.isChecked()) {
                Toast.makeText(mContext,  mContext.getResources().getString(R.string.DHCP_IPV6_HANLDER_MSG), Toast.LENGTH_LONG).show();
            } else if((mNetTypeIPv4.isChecked() || mNetTypeIPv6.isChecked())&& mConTypeManual.isChecked() && handle_saveconf()) {
                Toast.makeText(mContext, mContext.getResources().getString(R.string.ETHERNET_STATIC_SUCCESS), Toast.LENGTH_LONG).show();
                mDialog.dismiss();
            } else {
                Toast.makeText(mContext, mContext.getResources().getString(R.string.ETHERNET_STATIC_FAILURE), Toast.LENGTH_LONG).show();
                Slog.v(TAG, "Static IP Configure is failed");
            }
        }
    };

    /*IPV4 Validation*/
    private static boolean ipv4Validator(final String ip){
        Pattern pattern = Pattern.compile(PATTERN);
        Matcher matcher = pattern.matcher(ip);
        return matcher.matches();
    }

    /*Check whether address is numeric address or not. 
     It avoids leading 0's in the address*/
    private boolean validateIpConfigFields(String address) {
        try {
            InetAddress inetAddress = NetworkUtils.numericToInetAddress(address);
        } catch (IllegalArgumentException e) {
            return false;    
        } 
        return true;  
    }

    /*IPV6 Validation*/
    private static boolean ipv6Validator(String address) {
       try {
           java.net.Inet6Address.getByName(address);
           return true;
       } catch(Exception e) {
           return false;
       }
   }
    
    /*Static IP Configuration - Save Handler*/
    private boolean handle_saveconf() {
        EthernetDevInfo info = new EthernetDevInfo();
        info.setIfName(INTERFACE_NAME);
        String ipaddr = null, dns = null, gateway = null, mask = null, prefixLen = null;

        if (localLOGV) Slog.v(TAG, "Config device for eth0");
            
        if(mIpaddr.getText().toString().trim().length() != 0) {
            ipaddr = mIpaddr.getText().toString().trim();
        }

        if( mDns.getText().toString().trim().length() != 0) {
            dns = mDns.getText().toString().trim();
        } 
 
        if(mGw.getText().toString().trim().length() != 0) {
            gateway = mGw.getText().toString().trim();
        }

        /*IP Validation*/ 
        if(ipaddr == null || ipaddr.equals("")) {
            Toast.makeText(mContext, mContext.getResources().getString(R.string.EMPTY_IP), Toast.LENGTH_LONG).show();
            mIpaddr.requestFocus();
            return false;
        }
 
        if(ipv4Validator(ipaddr) && validateIpConfigFields(ipaddr)) {
            info.setNetType(ETHERNET_IPV4);
            if(mMask.getText().toString().trim().length() != 0) {  
                mask = mMask.getText().toString().trim();
            }
            /*NetMask Validation*/ 
	    if(!(mask == null || mask.equals(""))) {
	        if(ipv4Validator(mask) && validateIpConfigFields(mask)) {
		} else if(ipv6Validator(mask) && validateIpConfigFields(mask)) {
		} else {
		    Toast.makeText(mContext, mContext.getResources().getString(R.string.NETMASK_INVALID), Toast.LENGTH_LONG).show();
		    mMask.requestFocus();
		    return false; 
		}
	    }     
            info.setNetMask(mask);
            mMask.clearFocus();
        } else if(ipv6Validator(ipaddr) && validateIpConfigFields(ipaddr)) {
            info.setNetType(ETHERNET_IPV6);
            if(mPrefixLen.getText().toString().trim().length() != 0) {
                prefixLen = mPrefixLen.getText().toString().trim();
            } 
            int prefixlen;
            try {
                prefixlen = Integer.parseInt(prefixLen);
            } catch (NumberFormatException ex) {
                Slog.e(TAG, "NumberFormatException for prefixlength" + ex);  
                Toast.makeText(mContext, mContext.getResources().getString(R.string.EMPTY_PREFIXLENGTH), Toast.LENGTH_LONG).show();
		mPrefixLen.requestFocus();
                return false;
            }
	    if(prefixlen < 1 || prefixlen > 128) {
	        Toast.makeText(mContext, mContext.getResources().getString(R.string.INVALID_PREFIXLENGTH), Toast.LENGTH_LONG).show();
		mPrefixLen.requestFocus();
		return false;
	    } 
            info.setPrefixLength(prefixlen); 
        } else {
            Toast.makeText(mContext, mContext.getResources().getString(R.string.IP_INVALID), Toast.LENGTH_LONG).show();
            mIpaddr.requestFocus();
            return false;
        }  

        info.setIpAddress(ipaddr);

        /* DNS Validation*/
        if(!(dns == null || dns.equals(""))) {
             if(ipv4Validator(dns) && validateIpConfigFields(dns)) {
             }else if(ipv6Validator(dns) && validateIpConfigFields(dns)) {
             } else {
                Toast.makeText(mContext, mContext.getResources().getString(R.string.DNS_INVALID), Toast.LENGTH_LONG).show();
                mDns.requestFocus();
                return false; 
             }
        }     
        info.setDnsAddr(dns);

        /*Gateway Validation*/
        if(!(gateway == null || gateway.equals(""))) {
	    if(ipv4Validator(gateway) && validateIpConfigFields(gateway)) {
            }else if(ipv6Validator(gateway) && validateIpConfigFields(gateway)) {
	    } else {
	        Toast.makeText(mContext, mContext.getResources().getString(R.string.GATEWAY_INVALID), Toast.LENGTH_LONG).show();
		mGw.requestFocus();
		return false; 
	    }
        }    
	info.setRouteAddr(gateway);

        info.setConnectMode(EthernetDevInfo.ETHERNET_CONN_MODE_MANUAL);
        mIpaddr.clearFocus();
        mDns.clearFocus();
        mGw.clearFocus();

        return mEthManager.updateDevInfo(info);
    }

    /*
     * Thread class to handle the saved DHCP or Manunal COnfiguration in Async
     */
    class setDhcpIpThread extends Thread {
        public setDhcpIpThread() {
            super("setDhcpIpThread");
        }

        public void run() {
            updateDhcpIp();
        }
    }

    private void updateDhcpIp() {
        if(null != mEthManager) {
            mEthManager.setDefaultConf();
            mEthManager.clearIPAddress();
            mEthManager.resetInterface();
        }
    }

    /*Dhcp IP Configuration - Save Handler*/
    private void startThread() {
       new setDhcpIpThread().start();
    }
}
