/*
 * Copyright (C) 2010 The Android-x86 Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Yi Sun <beyounn@gmail.com>
 * 22-07-2013, Gigaset Communications GmbH
 * Feature added to configure ethernet ip
 
 */



package com.maxwell.settings.ethernet;


import static com.maxwell.net.ethernet.EthernetManager.ETHERNET_STATE_DISABLED;
import static com.maxwell.net.ethernet.EthernetManager.ETHERNET_STATE_ENABLED;
import static com.maxwell.net.ethernet.EthernetManager.ETHERNET_STATE_UNKNOWN;

import com.android.settings.R;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.NetworkInfo;
import com.maxwell.net.ethernet.EthernetManager;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceScreen;
import android.preference.CheckBoxPreference;
import android.text.TextUtils;
import android.util.Config;
import android.util.Slog;
import android.widget.Switch;
import android.widget.Toast;
import android.app.Activity;
import android.widget.CompoundButton;
import android.os.UserHandle;

public class EthernetEnabler implements CompoundButton.OnCheckedChangeListener {
    private static final String TAG = "SettingsEthEnabler";
    private static final boolean LOCAL_LOGD = false;
    private EthernetManager mEthManager;
    private Context mContext;
    private Switch mSwitch;
    private IntentFilter mFilter;
    private static final String PCWIFICLIENT_STATE = "com.maxwell.net.ethernet.PCWIFICLIENT_STATE";
    private static final String ETHSTATE_CHANGE = "com.maxwell.net.ethernet.ETHSTATE_CHANGE";

    private final BroadcastReceiver mReceiver = new BroadcastReceiver() {
        @Override
            public void onReceive(Context context, Intent intent) {
                if(intent != null) {
                    handleEvent(context, intent);
                }
            }
    };

    public EthernetEnabler(Context context, Switch switch_) {
        mContext = context;
        mSwitch = switch_;
        mEthManager = (EthernetManager) context.getSystemService(Context.ETHERNET_SERVICE);

        mFilter = new IntentFilter();
        mFilter.addAction(PCWIFICLIENT_STATE);
        mFilter.addAction(ETHSTATE_CHANGE);
    }
   
    /*
     * Set the Ethernet Switch based on the Ethernet State
     * if state is Enabled, then set switch to true
     * else set switch to false
     */
    public void setSwitch(Switch switch_) {
        if(LOCAL_LOGD) Slog.d(TAG, "setSwitch");

        if (mSwitch == switch_) return;

        mSwitch.setOnCheckedChangeListener(null);
        mSwitch = switch_;
        mSwitch.setOnCheckedChangeListener(this);

        final int EthState = mEthManager.getState();
        if(LOCAL_LOGD) Slog.d(TAG, "Eth state is" + EthState);
        boolean isEnabled = EthState == EthernetManager.ETHERNET_STATE_ENABLED;
        mSwitch.setChecked(isEnabled);
        updateSwitchState();
    }

    public EthernetManager getManager() {
        return mEthManager;
    }

    public void resume() {
        if(LOCAL_LOGD) Slog.d(TAG, "resume");
        mContext.registerReceiver(mReceiver, mFilter);
        mSwitch.setOnCheckedChangeListener(this);
    }

    public void pause() {
        if(LOCAL_LOGD) Slog.d(TAG, "pause");
		mContext.unregisterReceiver(mReceiver);
        mSwitch.setOnCheckedChangeListener(null);
    }

    public void destroy() {
        if(LOCAL_LOGD) Slog.d(TAG, "destroy");
		
    }

    /*
     * This will be called when ever user changes the Ethernet swiches in the UI
     */
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
        if(LOCAL_LOGD) Slog.d(TAG, "onCheckedChanged" + isChecked);

        if(isChecked) {
            if(EthernetManager.ETHERNET_STATE_PCPORT_ETHERNET_ENABLED ==  mEthManager.getState()) {
                mSwitch.setChecked(false);
                return;
            } else if(EthernetManager.ETHERNET_STATE_ENABLED !=  mEthManager.getState()){
                mEthManager.setEnabled(true);
            }
            setSwitchChecked(true);
            mSwitch.setEnabled(false);
        } else {
            setSwitchChecked(false);
            mSwitch.setEnabled(false);
            mEthManager.setEnabled(false);
        }
    }

    /*
     * setting the switch status
     */
    private void setSwitchChecked(boolean checked) {
        if(LOCAL_LOGD) Slog.d(TAG, "setSwitchChecked" + checked + mSwitch.isChecked());
        if (checked != mSwitch.isChecked()) {
            mSwitch.setChecked(checked);
        }
    }

    private void handleEvent(Context context, Intent intent) {
        String action = intent.getAction();
        if(PCWIFICLIENT_STATE.equals(action)) {
            boolean pCWifiClientstatus = intent.getExtras().getBoolean("PCWIFICLIENT_STATUS");
            updatePCWifClientStatus(pCWifiClientstatus);
        } else if (ETHSTATE_CHANGE.equals(action)) {
            updateSwitchState();
        }
    }

    private void updateSwitchState() {
        final int EthState = mEthManager.getState();
        final int IntState = mEthManager.getIntState();
        if(LOCAL_LOGD) Slog.d(TAG, "updateSwitchState" + EthState + IntState);

        switch (EthState) {
            case EthernetManager.ETHERNET_STATE_DISABLED:
                if(IntState == EthernetManager.ETHERNET_STATE_DISABLED) {
                    mSwitch.setEnabled(true); 
                } else if (IntState == EthernetManager.ETHERNET_STATE_DISABLING) {
                    mSwitch.setEnabled(false);
                } else {
                    Slog.e(TAG, "UnknownState");
                }
                break;

            case EthernetManager.ETHERNET_STATE_ENABLED:
                if(IntState == EthernetManager.ETHERNET_STATE_ENABLING) {
                    mSwitch.setEnabled(true);
                } else if(IntState == EthernetManager.ETHERNET_STATIC_IP_CONFIGURING) {
                    mSwitch.setEnabled(false);
                } else if(IntState == EthernetManager.ETHERNET_DHCP_IP_CONFIGURING) {
                    mSwitch.setEnabled(false);
                } else if(IntState == EthernetManager.ETHERNET_IP_CONF_SUCCESS) {
                    mSwitch.setEnabled(true);
                } else if(IntState == EthernetManager.ETHERNET_STATIC_IP_ERROR) {
                    mSwitch.setEnabled(true);
                } else if(IntState == EthernetManager.ETHERNET_DHCP_IP_ERROR) {
                    mSwitch.setEnabled(true);
                } else if(IntState == EthernetManager.ETHERNET_LINK_DOWN) {
                    mSwitch.setEnabled(true);
                } else {
                    Slog.e(TAG, "UnknownState");
                }
                break;

            case EthernetManager.ETHERNET_STATE_PCPORT_ETHERNET_ENABLED:
                mSwitch.setEnabled(true);
                break;

            case EthernetManager.ETHERNET_STATE_UNKNOWN:  
                mSwitch.setEnabled(true);
                break;
        }
    }

    private void updatePCWifClientStatus(boolean isPCWifiClientStatus) {
        if (mContext == null) return;

        if(LOCAL_LOGD) Slog.d(TAG, "updatePCWifClientStatus" + isPCWifiClientStatus);

        if(isPCWifiClientStatus) {
            setSwitchChecked(false);
            mSwitch.setEnabled(true);
        } else {
            setSwitchChecked(true);
            mSwitch.setEnabled(true);
        }
    }
}
