/*
This program is distributed under the terms of the 'MIT license'. The text
of this licence follows...

Copyright (c) 2006,2007 J.D.Medhurst (a.k.a. Tixy)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/**
@file

@brief Command-line progran for Y-Modem file transfer.
*/

#include "common.h"
#include "ymodem_tx.h"
#include "ymodem_rx.h"
#include "dvf99_upgrade.h"


#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <cutils/properties.h>





#ifdef LINUX
#define DEFAULT_PORT 0							/**< Default value for #PortNum */
#else
#define DEFAULT_PORT 1
#endif
#define DEFAULT_BAUD 115200						/**< Default value for #PortBaud */
#define DEFAULT_TIMEOUT 30						/**< Default value (in seconds) for #Timeout */
#define DEFAULT_LOG_TIMEOUT 10					/**< Default value (in seconds) for #LogTimeout */
#define MAX_LOG_END_STRING_LENGTH 128			/**< Maximum length for #LogEndString */


#define STRINGIFY2(a) #a						/**< Helper for #STRINGIFY */
#define STRINGIFY(a) STRINGIFY2(a)				/**< Convert \a a into a quoted string */


const char* ProgramName;						/**< Name of this program */
unsigned PortNum = DEFAULT_PORT;				/**< Serial port number to use */
unsigned PortBaud = DEFAULT_BAUD;				/**< Baud rate for serial port */
unsigned Timeout = DEFAULT_TIMEOUT*1000;		/**< Timeout (in milliseconds) before aborting transfer */
bool XModemFlag = false;						/**< True for X-Modem, false for Y-Modem */
bool CrcFlag = false;							/**< True for X-Modem CRC mode */
bool KModeFlag = false;
bool GModeFlag = false;							/**< True for Y-Modem G mode */
bool SendFlag = false;							/**< True if sending file */
bool ReceiveFlag = false;						/**< True if receiving file */
const char* FileName = 0;						/**< Name of file to be transferred */
const char* DstFileName = 0;					/**< File name to send to receiver in Y-Modem transer */
const char* LogFileName = 0;					/**< Name of file for 'capture log' */
unsigned LogTimeout = DEFAULT_LOG_TIMEOUT*1000;	/**< Timeout (in milliseconds) before aborting 'capture log' */
const char* LogEndString = 0;					/**< String to terminate 'capture log' */
size_t LogEndStringLength = 0;					/**< Length of #LogEndString */
bool LogEcho = false;							/**< True if log should be echoed to STDOUT */
size_t FileSize;


/**
Error values specifiec to this program.
*/
enum Error
	{
	ErrorNoMemory		= -500, 				/**< Insufficient memory */
	ErrorLogFileError	= -501					/**< Error occured creating or writing to 'capture log' */
	};


/**
Print message when a error is detected.
Then do <code> exit(error); </code>.

@param error	The error number.
@param format	Format string a la printf.
@param ...		Arguments for format sting, a la printf.
*/
void Dvf99_upgrade::Error(int error,const char *format, ...)
	{
	va_list args;
	va_start(args,format);
	vfprintf(stderr,format,args);
	va_end(args);
	if(error<0)
		fprintf(stderr,". Error %d",error);
	fprintf(stderr,"\n");
	exit(error);
	}



/**
Class for presenting a file as a input stream.
*/
class InFile : public YModemTx::InStream
	{
public:
	InFile()
		: File(0)
		{
		}

	/**
	Open stream for reading a file.

	@param fileName		Name of the file to read.

	@return Zero if successful, or a negative error value if failed.
	*/
	int Open(const char* fileName)
		{
		File = fopen(fileName,"rb");
		if(!File)
			return YModemTx::ErrorInputStreamError;
		// find file size...
		if(fseek(File,0,SEEK_END))
			{
			fclose(File);
			return YModemTx::ErrorInputStreamError;
			}
		TotalSize = ftell(File);
		if(fseek(File,0,SEEK_SET))
			{
			fclose(File);
			return YModemTx::ErrorInputStreamError;
			}
		TransferredSize = 0;
		return 0;
		}

	/**
	Close the stream.
	*/
	void Close()
		{
		if(File)
			{
			fclose(File);
			File = 0;
			}
		}

	/**
	Return the size of the file.

	@return File size.
	*/
	inline size_t Size()
		{
		return TotalSize;
		}

	/**
	Read data from the stream.

	@param[out] data	Pointer to buffer to hold data read from stream.
	@param size			Maximum size of data to read.

	@return Zero if successful, or a negative error value if failed.
	*/
	int In(uint8_t* data, size_t size)
		{

			

		
		
		int percent = TotalSize ? ((uint64_t)TransferredSize*(uint64_t)100)/(uint64_t)TotalSize : 0;
		
		
		//fprintf(stderr,"%8d bytes of %d - %3d%%\r",TransferredSize, TotalSize, percent);
		
		fflush(stdout);

		__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","InFile In TotalSize :%d  TransferredSize:%d  percent:%d \n",TotalSize, TransferredSize, percent);

		size=fread(data,sizeof(uint8_t),size,File);

		
		if(size)
			{
			TransferredSize += size;
			return size;
			}
		if(TransferredSize!=TotalSize)
			return YModemTx::ErrorInputStreamError;
		return 0;
		}
private:
	FILE* File;
	size_t TotalSize;
	size_t TransferredSize;
	};


/**
Class for presenting a file as a output stream.
*/
class OutFile : public YModemRx::OutStream
	{
public:
	OutFile()
		: File(0)
		{
		}

	/**
	Open stream for writing to a file.

	If no data has yet been written to the stream, this function may be
	called a second time in order to update the file size.

	@param fileName		Name of the file to write.
	@param size			Size of data being written to a file.

	@return Zero if successful, or a negative error value if failed.
	*/
	int Open(const char* fileName, size_t size)
		{
		if(File)
			{
			if(TransferredSize)
				return YModemRx::ErrorOutputStreamError;
			}
		else
			{
			File = fopen(fileName,"w+b");
			if(!File)
				return YModemRx::ErrorOutputStreamError;
			printf("Receiving %s...\n",fileName);
			}
		TotalSize = size;
		TransferredSize = 0;
		return 0;
		}

	/**
	Close the stream.
	*/
	void Close()
		{
		if(File)
			{
			fclose(File);
			File = 0;
			}
		}

	/**
	Write data to the stream.

	@param data		Pointer to data to write.
	@param size		Size of data.

	@return Zero if successful, or a negative error value if failed.
	*/
	int Out(const uint8_t* data, size_t size)
		{
		if(!File)
			return YModemRx::ErrorOutputStreamError; // we've not been Open()ed
		// show progress...
		if(TotalSize)
			{
			int percent = TotalSize ? ((uint64_t)TransferredSize*(uint64_t)100)/(uint64_t)TotalSize : 0;
			printf("%8d bytes of %d - %3d%%\r",TransferredSize, TotalSize, percent);
			fflush(stdout);
			}
		else
			{
			printf("%8d bytes\r",TransferredSize);
			fflush(stdout);
			}
		// limit data to size of file...
		if(TotalSize)
			{
			size_t maxSize = TotalSize-TransferredSize;
			if(size>maxSize)
				size = maxSize;
			if(!maxSize)
				return 0; // end
			}
		// write data...
		if(size!=fwrite(data,sizeof(uint8_t),size,File))
			return YModemRx::ErrorOutputStreamError; // write failed
		TransferredSize += size;
		return size;
		}
private:
	FILE* File;
	size_t TotalSize;
	size_t TransferredSize;
	};


FILE* LogFile = 0;	/**< Handle of the 'capture log' file */






/**
Send the file.

@param port	The serial port to use.
*/
void Dvf99_upgrade::Send(SerialPort* port)
	{		

	__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","File to be sent  : %s\n", FileName);


	InFile source;
	int error = source.Open(FileName);
	if(error)
		Error(error,"Can't open file '%s'",FileName);

	printf("Sending File via YModem :%s\n",FileName);
	FileSize = source.Size();
	printf("Size of the file before transfer :%d\n",FileSize);

	
	DstFileName = FileName;
	
	YModemTx ymodem(*port);	
	
	error = ymodem.SendY(DstFileName,source.Size(),source,Timeout);
	if(error)
		Error(error,"Error during file transfer");

	printf("\nSent OK\n");
	
	}


/**
Receive a file.

@param port	The serial port to use.
*/
void Dvf99_upgrade::Receive(SerialPort* port)
	{

	__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade",">> Receive ");
	OutFile destination;
	YModemRx ymodem(*port);
	int error;
	if(XModemFlag)
		{		
		error = destination.Open(FileName,0);
		if(error)
			Error(error,"Can't create file '%s'",FileName);
		error = ymodem.ReceiveX(destination,Timeout,CrcFlag);
		}
	else
		{
		if(FileName)
			{

			__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade",">> Receive Ymodem file s:%s",FileName);
			error = destination.Open(FileName,0);
			if(error)
				Error(error,"Can't create file '%s'",FileName);
			}
		error = ymodem.ReceiveY(destination,Timeout,GModeFlag);
		}
	if(error)
		Error(error,"Error during file transfer");

	printf("\nReceived OK\n");
	destination.Close();
	}


/**
Main function.

@param argc		The number of arguments.
@param argv		Array of argument strings.

@return Zero if transfer succeeded.
*/
int Dvf99_upgrade::upgrade_DVF99(const char *portName, unsigned baudRate, const char *sourceFile)
	{


	__android_log_print(ANDROID_LOG_VERBOSE,"Dvf99_upgrade", "upgrade_DVF99 params:\n PortName :%s\n baudRate:%d\n sourceFile :%s\n ",portName, baudRate, sourceFile);
	

	SerialPort* port = SerialPort::New();
	if(!port)
	{
		__android_log_print(ANDROID_LOG_ERROR,"Dvf99_upgrade", "Error opening the port");
		return -1;
	}

	int error = port->Open(portName);
	if(error)
	{
		__android_log_print(ANDROID_LOG_ERROR,"Dvf99_upgrade", "Error opening the port");
		delete port;
		return -1;
	}

	error = port->Initialise(baudRate);
	if(error)
	{
		__android_log_print(ANDROID_LOG_ERROR,"Dvf99_upgrade", "Error initialising the port");
		port->Close();
		delete port;
		return -1;
	}

	const char *ctrlc = "\003";
    const char  *cmd = "run upgrade \015";	


	FileName = sourceFile;		


	int result = port->Out((uint8_t *)ctrlc,strlen(ctrlc),1000);

	
	result = port->Out((uint8_t *)cmd,strlen(cmd),1000);

	const char *ymodem = "Ready for binary (ymodem) download";

	result = readDVFPort(port,ymodem,2000);

	if(result < 0)
	{
		__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","Search string NOT FOUND, reading failed");
		port->Close();
		delete port;
		return -1;
	}	

	//Send file to port
	Send(port);	

	uint8_t c = 0;
	result = port->In(&c,1,1000);	
	
	
	char txResult[1024];
	int j = 0;
	uint8_t s = 0;
	

	while(result > 0)
	{		
		result = port->In(&s,1,30000);				
		txResult[j] = s;
		j++;
	}
	txResult[j] = '\0';

	__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","final output after transfer :%s \n",txResult);

	
	char bytes[40];

	sprintf(bytes, "%zu%s",FileSize, " Bytes");

	const char *text = "Copy to Flash... done";

	char *find = strstr(txResult, text);
	char *txbytes = strstr(txResult, bytes);

	__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","search string find :%s \n",find);
	__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","search string txbytes :%s \n",txbytes);

	if(find == NULL || txbytes == NULL)
	{
		port->Close();
		delete port;
		return -1;
	} 	

	const char  *erasecss = "erase css \015";

	result = port->Out((uint8_t *)erasecss,strlen(erasecss),1000);	

	const char *css = "Erased";

	result = readDVFPort(port,css,20000);

	if(result < 0)
	{
		__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","Search string NOT FOUND, reading failed");
		port->Close();
		delete port;
		return -1;
	}	

	
	char cpcmd[100] = {0,};
	strcpy(cpcmd,"cp.b ${loadaddr} css ");
	
	memset(bytes,0x00,40*sizeof(char));
	sprintf(bytes, "0x%x%s",FileSize," \015");
	
	strcat(cpcmd, bytes);

	result = port->Out((uint8_t *)cpcmd,strlen(cpcmd),1000);

	const char *cp = "Copy to Flash... done";

	result = readDVFPort(port,cp,10000);

	if(result < 0)
	{
		__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","Search string NOT FOUND, reading failed");
		port->Close();
		delete port;
		return -1;
	}


	char setenv[200] = {0,};
	strcpy(setenv,"setenv bootm \"run chk_parts; cp.b css ${loadaddr} && css load ${loadaddr} ");	
	memset(bytes,0x00,40*sizeof(char));
	sprintf(bytes, "0x%x%s",FileSize,"\" \015");
	
	strcat(setenv, bytes);

	result = port->Out((uint8_t *)setenv,strlen(setenv),1000);

	const char	*saveenv = "saveenv \015"; 
	  	   
	result = port->Out((uint8_t *)saveenv,strlen(saveenv),1000);

	const char *save = "Writing to Flash... done";

	result = readDVFPort(port,save,20000);

	if(result < 0)
	{
		__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","Search string NOT FOUND, reading failed");
		port->Close();
		delete port;
		return -1;
	}	

	port->Close();
	delete port;

	return 0;
	}


	int Dvf99_upgrade::readDVFPort(SerialPort* port, const char * text,  unsigned timeout)
	{
		char output[1024];
		int offset = 0;
		uint8_t buffer[1024];
		char temp[1024];
		memset(buffer,0,sizeof(buffer));
		memset(temp,0,sizeof(temp));

		int result = 1;

		__android_log_print(ANDROID_LOG_ERROR,"DVF99_upgrade", "Search string is :%s", text);
		

		while(result > 0)
		{ 	  
			result = port->In(buffer+offset,sizeof(buffer) - (offset+1),timeout);			

			if(result>0) {			  	 			   
				offset += result;

				memcpy(temp,buffer,offset);
				temp[offset+1] = '\0';

				char *find = strstr(temp+1, text);
				if(find != NULL)
				{
					__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","search string found returning success:%s \n",find);
					memset(temp,0,sizeof(temp));			    
					return 0;
				}
				memset(temp,0,sizeof(temp));
			}
		}	   

		memcpy(output,buffer,offset);	  
		output[offset] = '\0';

		__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","run bootm output is :%s \n", output+1);

		char *search = strstr(output+1, text);

		__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","search string :%s \n",search);

		if(search == NULL)
		{
			__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","search string :%s \n",search);		

			if(!strcmp(text, "CSS firmware successfully loaded"))
			{
				__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","Bootm command from CMBS case");
				
				char *find = strstr(output+1, "invalid image");

				if(find != NULL)
				{
					__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","Invalid image magic error from dvf side");
					return -200;
				}
			}
				
			return -1;
		}

		return 0;
	}
			


	/* Fucntion to boot the DVF99 module by sending the "run bootm" command*/
	int Dvf99_upgrade::bootDVF99Module()
	{

		SerialPort* port = SerialPort::New();
		if(!port)
		{
			__android_log_print(ANDROID_LOG_ERROR,"DVF99_upgrade", "Error while creating serial port");
			return -1;
		}

		int error = port->Open("/dev/ttymxc2");
		if(error)
		{
			__android_log_print(ANDROID_LOG_ERROR,"DVF99_upgrade", "Error opening the port");
			delete port;
			return -1;
		}

		error = port->Initialise(115200);
		if(error)
		{
			__android_log_print(ANDROID_LOG_ERROR,"DVF99_upgrade", "Error initialising the port");
			port->Close();
			delete port;
			return -1;
		}

		const char *ctrlc = "\003";
		const char  *cmd = "run bootm \015";

		int result = port->Out((uint8_t *)ctrlc,strlen(ctrlc),1000);

		result = port->Out((uint8_t *)cmd,strlen(cmd),1000);

		usleep(1*1000*1000);

		const char *word = "CSS firmware successfully loaded";

		result = readDVFPort(port,word,8000);

		if(result < 0)
		{
			__android_log_print(ANDROID_LOG_VERBOSE, "DVF99_upgrade","Search string NOT FOUND, reading failed");
			port->Close();
			delete port;
			return result;
			
		}

	    if(port != NULL)
	    {
	    	port->Close();
			delete port;
	    }
		
	    return 0;

	}
		
		
	/* Fucntion to reset the DVF99 module by GPIO reset*/
	void Dvf99_upgrade::doGPIOReset(){
		__android_log_print(ANDROID_LOG_VERBOSE,"DVF99_upgrade","Initiating GPIO reset of the DVF99");
		property_set("ctl.start", "gpioreset_script");
	}


	Dvf99_upgrade::Dvf99_upgrade()
	{
		
	}


	Dvf99_upgrade::~Dvf99_upgrade()
	{
	}



/** @} */ // End of group

